package it.unimi.evotion.tasks;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;

import org.apache.commons.io.FileUtils;
import org.apache.spark.SparkContext;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.mllib.clustering.KMeans;
import org.apache.spark.mllib.clustering.KMeansModel;
import org.apache.spark.mllib.linalg.Vector;
import org.apache.spark.mllib.linalg.Vectors;
import org.apache.spark.rdd.RDD;
import org.apache.spark.sql.SparkSession;

public class Kmeans implements Task,Serializable{

	
	
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JavaRDD<Vector> parsedData;
	private KMeansModel clusters;
	private RDD<Vector> parsedRdd;
	
	/**
     * Check if is a vector of numbers
     *
     * @param s
     * @return
     */
    public static boolean isVector(String s) {
        String[] sarray = s.split(",");
        try {
            Double.parseDouble(sarray[0]);
            return true;
        }
        catch (Exception e) {
            return false;
        }
    }

    /**
     * Convert in a vector of numbers (double)
     *
     * @param s
     * @return
     */
    public static Vector asVector(String s) {
        String[] sarray = s.split(",");
        double[] values = new double[sarray.length];
        for (int i = 0; i < sarray.length; i++) {
            values[i] = Double.parseDouble(sarray[i]);
        }
        return Vectors.dense(values);
    }

    /**
     * Delete the directory
     *
     * @param dir
     */
    static void deleteDir(String dir) {
        try {
            FileUtils.deleteDirectory(new File(dir));
        } catch (IOException e) {

        }
    }

    /**
     * Concatenate two arrays
     *
     * @param u
     * @param v
     * @return
     */
    static double[] concat(double[] u, double[] v) {
        double[] r = new double[u.length + v.length];
        int i = 0;

        for (double a : u)
            r[i++] = a;
        for (double a : v)
            r[i++] = a;

        return r;
    }

    /**
     * Convert the array in string
     *
     * @param a
     * @return
     */
    private static String toString(double[] a) {
        if (a == null || a.length == 0)
            return "";

        StringBuffer sb = new StringBuffer();
        sb.append(a[0]);

        for(int i=1; i<a.length; ++i)
            sb.append(",").append(a[i]);

        return sb.toString();
    }

    public void init(Object...params) {
    	SparkContext sc =((SparkSession) params[0]).sparkContext();
    	
    	// read the file
        JavaRDD<String> data  = sc.textFile((String)params[1], 1).toJavaRDD();

        // remove the CSV header
        data = data.filter(s -> isVector(s));

        // convert the line in a vector of doubles
         parsedData = data.map(s -> asVector(s));
    }
    
    public void run(Object...params) {
    	
    	 int numClusters = (Integer)params[0];
         int numIterations =  (Integer)params[1];
         parsedRdd = parsedData.rdd();
         clusters = KMeans.train(parsedRdd, numClusters, numIterations);
    }
    
       public void postProcessing(Object...params) {
    	//
        // predict the cluster of the records in the rdd
        //
    	JavaRDD<Object> objCluster = clusters.predict(parsedRdd).toJavaRDD();
        JavaRDD<Vector> vctCluster = objCluster.map(o -> { return Vectors.dense(((Integer)o).doubleValue()); });

        //
        // concatenate two rdd in horizontal
        //
        JavaRDD<Vector> inCluster = parsedData
            .zip(vctCluster)
            .map(t -> {
                return Vectors.dense(concat(t._1().toArray(), t._2().toArray()));
            });

        //
        // generate the csv line
        //
        JavaRDD<String> csvout = inCluster
            .map(v -> {
                double[] a = v.toArray();
                return toString(a);
            });


        // save the result
        csvout
            .saveAsTextFile((String)params[1]);

        ((SparkSession)params[0]).close();
    	
    }
    
    public static void main(String[] args) {
    	
        SparkSession spark = SparkSession
            .builder()
            .appName("Kmeans")
            .getOrCreate();

        deleteDir(args[1]);

        Kmeans kmeans=new Kmeans();
        kmeans.init(spark,args[0]);
        kmeans.run(Integer.parseInt(args[2]),Integer.parseInt(args[3]));
        kmeans.postProcessing(spark,args[1]);
        
      
        
    }
}
